@@ -1,5 +1,6 @@
 from . import *
 from time import time
+
 def _build_bfrc(cls):
     #t = time()
     for atom0, c in cls.connectivity.items():
@@ -28,9 +29,14 @@ def _build_bfrc(cls):
                     for backup in backups[i-1]:
                         good_backup = True
                         for j, atomj in enumerate(backup):
-                            if atomj == atom1 or top_matrix[j][i] <= 1 or atom1 not in atomj.linked_atoms[top_matrix[j][i]]:
+                            if atomj == atom1 or abs(top_matrix[j][i]) <= 1 or atom1 not in atomj.linked_atoms[abs(top_matrix[j][i])]:
                                 good_backup = False
                                 break
+                            if top_matrix[j][i] <= -1:
+                                for d2 in range(2, d):
+                                   if atom1 in atomj.linked_atoms[d2]:
+                                    good_backup = False
+                                    break
                         if  good_backup:
                             backups[i].append([*backup,atom1])
             frc_all.extend(backups[len(top)-1])
@@ -67,7 +73,8 @@ def _build_bfrc(cls):
                         tofindname = "-".join(p) 
                         if tofindname in frc.types.keys():
                             finded[tofindname] = [frc.types[tofindname], frc_one]
-                            break                         
+                            break
+                   
             
             assert (not frc.compulsory or len(finded) == 1), "None of %s type found for %s"%(frc.name, "-".join([atom.type.name for atom in frc_one]))
             
@@ -114,24 +121,33 @@ def _build_bfrc_link(cls):
     atom2_friends = set([atom2])
     
     far = GlobalSetting.farthest_bonded_force
-    #t = time()
+    temp_atom1_linked = {i : [] for i in range(far, 2, -1)}
+    temp_atom2_linked = {i : [] for i in range(far, 2, -1)}
+    #t = time()    
     for i in range(far-1, 1, -1):
         for atom in atom1.linked_atoms[i]:
             atom.linked_atoms[i+1].append(atom2)
-            atom2.linked_atoms[i+1].append(atom)
+            temp_atom2_linked[i+1].append(atom)
             atom1_friends.add(atom)
         for atom in atom2.linked_atoms[i]:
             atom.linked_atoms[i+1].append(atom1)
-            atom1.linked_atoms[i+1].append(atom)
+            temp_atom1_linked[i+1].append(atom)
             atom2_friends.add(atom)
+    for i in range(far-1, 1, -1):
+        atom1.linked_atoms[i+1].extend(temp_atom1_linked[i+1])
+        atom2.linked_atoms[i+1].extend(temp_atom2_linked[i+1]) 
     atom1.linked_atoms[2].append(atom2)
     atom2.linked_atoms[2].append(atom1)
+
+    
     for i in range(2, far):
         for j in range(2, far + 1 - i):
             for atom1_linked_atom in atom1.linked_atoms[i]:
                 for atom2_linked_atom in atom2.linked_atoms[j]:
-                    atom1_linked_atom.linked_atoms[i+j].append(atom2_linked_atom)
-                    atom2_linked_atom.linked_atoms[j+i].append(atom1_linked_atom)
+                    if atom1_linked_atom not in atom2_friends and atom2_linked_atom not in atom1_friends:
+                        atom1_linked_atom.linked_atoms[i+j].append(atom2_linked_atom)
+                        atom2_linked_atom.linked_atoms[j+i].append(atom1_linked_atom)
+
     #print("analysis of connectivity: %f"%(time()-t))
     atom12_friends = atom1_friends | atom2_friends
     for frc in GlobalSetting.BondedForces:
@@ -149,9 +165,14 @@ def _build_bfrc_link(cls):
                     for backup in backups[i-1]:
                         good_backup = True
                         for j, atomj in enumerate(backup):
-                            if atomj == atom1 or atomj not in atom12_friends or top_matrix[j][i] <= 1 or atom1 not in atomj.linked_atoms[top_matrix[j][i]]:
+                            if atomj == atom1 or abs(top_matrix[j][i]) <= 1 or atom1 not in atomj.linked_atoms[abs(top_matrix[j][i])]:
                                 good_backup = False
                                 break
+                            if top_matrix[j][i] <= -1:
+                                for d2 in range(2, d):
+                                   if atom1 in atomj.linked_atoms[d2]:
+                                    good_backup = False
+                                    break
                         if  good_backup:
                             backups[i].append([*backup,atom1])
             for backup in backups[len(top)-1]:
@@ -191,7 +212,7 @@ def _build_bfrc_link(cls):
                         tofindname = "-".join(p) 
                         if tofindname in frc.types.keys():
                             finded[tofindname] = [frc.types[tofindname], frc_one]
-                            break                    
+                            break
 
             assert (not frc.compulsory or len(finded) == 1), "None of %s type found for %s"%(frc.name, "-".join([atom.type.name for atom in frc_one]))
             
@@ -225,7 +246,7 @@ def Build_Bonded_Force(cls):
             cls.builded = True
     
         
-        
+
  
     else:
         raise NotImplementedError
@@ -234,36 +255,50 @@ def Build_Bonded_Force(cls):
 
 
 def Save_SPONGE_Input(molecule, prefix = None, dirname = "."):
-    Build_Bonded_Force(molecule)
-    
-    if not prefix:
-        prefix = molecule.name
-    
-    molecule.atoms = []
-    molecule.bonded_forces = { frc.name:[] for frc in GlobalSetting.BondedForces}
-    for res in molecule.residues:
-        molecule.atoms.extend(res.atoms)
-        for frc in GlobalSetting.BondedForces:
-            molecule.bonded_forces[frc.name].extend(res.bonded_forces[frc.name])
-            
-    for link in molecule.residue_links:
-        for frc in GlobalSetting.BondedForces:
-            molecule.bonded_forces[frc.name].extend(link.bonded_forces[frc.name])
-            
-    molecule.atom_index = { molecule.atoms[i]: i for i in range(len(molecule.atoms))}
-    
-    for vatom_type_name, vatom_type_atom_numbers in GlobalSetting.VirtualAtomTypes.items():
-        for vatom in molecule.bonded_forces[vatom_type_name]:
-            this_vatoms = [vatom.atoms[0]]
-            for i in range(vatom_type_atom_numbers):
-                this_vatoms.append(molecule.atoms[molecule.atom_index[vatom.atoms[0]] + getattr(vatom, "atom%d"%i)]) 
-            this_vatoms.sort(key = lambda x:molecule.atom_index[x])
-            while this_vatoms:
-                tolink = this_vatoms.pop(0)
-                for i in this_vatoms:
-                    if "v" not in tolink.linked_atoms.keys():
-                        tolink.linked_atoms["v"] = []
-                    tolink.linked_atoms["v"].append(i) 
-                    
-    for func in Molecule.save_functions:
-        func(molecule, prefix, dirname)
\ No newline at end of file
+    if type(molecule)== Molecule:
+        Build_Bonded_Force(molecule)
+        
+        if not prefix:
+            prefix = molecule.name
+        
+        molecule.atoms = []
+        molecule.bonded_forces = { frc.name:[] for frc in GlobalSetting.BondedForces}
+        for res in molecule.residues:
+            molecule.atoms.extend(res.atoms)
+            for frc in GlobalSetting.BondedForces:
+                molecule.bonded_forces[frc.name].extend(res.bonded_forces[frc.name])
+                
+        for link in molecule.residue_links:
+            for frc in GlobalSetting.BondedForces:
+                molecule.bonded_forces[frc.name].extend(link.bonded_forces[frc.name])
+                
+        molecule.atom_index = { molecule.atoms[i]: i for i in range(len(molecule.atoms))}
+        
+        for vatom_type_name, vatom_type_atom_numbers in GlobalSetting.VirtualAtomTypes.items():
+            for vatom in molecule.bonded_forces[vatom_type_name]:
+                this_vatoms = [vatom.atoms[0]]
+                for i in range(vatom_type_atom_numbers):
+                    this_vatoms.append(molecule.atoms[molecule.atom_index[vatom.atoms[0]] + getattr(vatom, "atom%d"%i)]) 
+                this_vatoms.sort(key = lambda x:molecule.atom_index[x])
+                while this_vatoms:
+                    tolink = this_vatoms.pop(0)
+                    for i in this_vatoms:
+                        if "v" not in tolink.linked_atoms.keys():
+                            tolink.linked_atoms["v"] = []
+                        tolink.linked_atoms["v"].append(i) 
+                        
+        for func in Molecule.save_functions:
+            func(molecule, prefix, dirname)
+
+    elif type(molecule) == Residue:
+        mol = Molecule(name = molecule.name)
+        mol.Add_Residue(molecule)
+        Save_SPONGE_Input(mol)
+    elif type(molecule) == ResidueType:
+        residue = Residue(molecule, name = molecule.name)
+        for atom in molecule.atoms:
+            residue.Add_Atom(atom.name, x = atom.x, y = atom.y, z = atom.z)
+        Save_SPONGE_Input(residue)
+
+sys.modules['__main__'].__dict__["Save_SPONGE_Input"] = Save_SPONGE_Input 
+
