@@ -22,6 +22,8 @@ import os
 import numpy as np
 import types
 from itertools import product, permutations
+import time
+import sys
 
 ##########################################################################
 #Basic Classes
@@ -37,9 +39,8 @@ class _GlobalSetting():
     UnitMapping = {"distance": {"nm": 1e-9, "A": 1e-10},
                    "energy"  : {"kcal/mol":4.184, "eV":96.4853, "kJ/mol": 1},
                    "charge"  : {"e":1,   "SPONGE":1.0/18.2223},
-                   "angle"   : {"degree":3.141592654,  "rad": 180}
-                   
-                    }
+                   "angle"   : {"degree":np.pi,  "rad": 180}
+                  }
     PDBResidueNameMap = {"head" : {}, "tail": {}}
     HISMap = {"DeltaH": "", "EpsilonH": "",   "HIS": {}}
     @staticmethod
@@ -217,7 +218,7 @@ class ResidueType(Type):
     @head.setter
     def head(self, atom):
         self.connect_atoms["head"] = atom
-
+        
     @property
     def tail(self):
         return self.connect_atoms["tail"]
@@ -226,6 +227,48 @@ class ResidueType(Type):
     def tail(self, atom):
         self.connect_atoms["tail"] = atom
         
+    @property
+    def tail_second(self):
+        return self.connect_atoms["tail_second"]
+    
+    @tail_second.setter
+    def tail_second(self, atom):
+        self.connect_atoms["tail_second"] = atom
+        
+    @property
+    def tail_third(self):
+        return self.connect_atoms["tail_third"]
+    
+    @tail_third.setter
+    def tail_third(self, atom):
+        self.connect_atoms["tail_third"] = atom
+
+    @property
+    def tail_bond(self):
+        return self.connect_atoms["tail_bond"]
+    
+    @tail_bond.setter
+    def tail_bond(self, atom):
+        self.connect_atoms["tail_bond"] = atom
+        
+    @property
+    def tail_angle(self):
+        return self.connect_atoms["tail_angle"]
+    
+    @tail_angle.setter
+    def tail_angle(self, atom):
+        self.connect_atoms["tail_angle"] = atom
+        
+    @property
+    def tail_dihedral(self):
+        return self.connect_atoms["tail_dihedral"]
+    
+    @tail_dihedral.setter
+    def tail_dihedral(self, atom):
+        self.connect_atoms["tail_dihedral"] = atom
+
+       
+      
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.atoms = []
@@ -236,7 +279,8 @@ class ResidueType(Type):
         self.builded = False
         self.link = {}
         self.bonded_forces = {frc.name:[] for frc in GlobalSetting.BondedForces}
-        self.connect_atoms = {"head": None, "tail":None}
+        self.connect_atoms = {"head": None, "tail": None, "tail_second":None, "tail_third":None, 
+                              "tail_bond":2, "tail_angle":109.5, "tail_dihedral": 180}
         
     def Add_Atom(self, name, atom_type, x, y, z):
         new_atom = Atom(atom_type, name)
@@ -290,7 +334,7 @@ class Entity():
         self.contents = {**entity_type.contents}
         self.count = type(self).count
         if not name:
-            name = entity_type.name + str(self.count)
+            name = entity_type.name
         type(self).count += 1
         self.name = name
         self.type = entity_type
@@ -310,11 +354,25 @@ class Atom(Entity):
     def __init__(self, entity_type, name = None):
         super().__init__(entity_type, name)
         self.linked_atoms = { i+1 : [] for i in range(1,GlobalSetting.farthest_bonded_force)}
+        self.residue = None
+        self.copied = {}
+    def deepcopy(self, forcopy = None):
+        new_atom = Atom(self.entity)
+        new_atom.contents = {**self.contents}
+        if forcopy:
+            self.copied[forcopy] = new_atom
+        return new_atom
 
 
 class Residue(Entity):
     name = "Residue"
     count = 0
+    
+    def __getattribute__(self, attr):
+        if attr not in  ("_name2atom", "contents") and attr in self._name2atom.keys():
+            return self._name2atom[attr]
+        else:
+            return super().__getattribute__(attr)
     def __init__(self, entity_type, name = None):
         super().__init__(entity_type, name)
         self.atoms = []
@@ -325,19 +383,32 @@ class Residue(Entity):
         self.builded = False
     
     def Add_Atom(self, name, atom_type = None, x = None, y = None, z = None):
-        if not atom_type: 
-            atom_type = self.type._name2atom[name].type
-            new_atom = Atom(atom_type, name)
-            new_atom.contents = self.type._name2atom[name].contents
+        if type(name) == Atom:
+            assert atom_type == None
+            new_atom = Atom(name.type, name.name)
+            new_atom.contents = {**name.contents}
+            name = name.name
         else:
-            new_atom = Atom(atom_type, name)
+            if not atom_type: 
+                atom_type = self.type._name2atom[name].type
+                new_atom = Atom(atom_type, name)
+                new_atom.contents = {**self.type._name2atom[name].contents}
+            else:
+                new_atom = Atom(atom_type, name)
+                new_atom.contents = {**self.type._name2atom[name].contents}
+        
+        new_atom.residue = self
         self.atoms.append(new_atom)
-        new_atom.x = float(x)
-        new_atom.y = float(y)
-        new_atom.z = float(z)
+        if x:
+            new_atom.x = float(x)
+        if y:
+            new_atom.y = float(y)
+        if z:
+            new_atom.z = float(z)
         self._name2atom[name] = new_atom
         self._atom2name[new_atom] = name
         self.connectivity[new_atom] = set([])
+        
     def Add_Connectivity(self, atom0, atom1):
         if type(atom0) == str:
             atom0 = self._name2atom[atom0]
@@ -352,6 +423,15 @@ class Residue(Entity):
         if type(bonded_force_entity).name not in self.bonded_forces.keys():
             self.bonded_forces[type(bonded_force_entity).name] = []
         self.bonded_forces[type(bonded_force_entity).name].append(bonded_force_entity)
+    
+    def deepcopy(self, forcopy = None):
+        new_residue = Residue(self.type)
+        for atom in self.atoms:
+            new_residue.Add_Atom(atom)
+            if forcopy:
+                atom.copied[forcopy] = new_residue.atoms[-1]
+
+        return new_residue
 
 class ResidueLink():
     def __repr__(self):
@@ -367,6 +447,9 @@ class ResidueLink():
         if type(bonded_force_entity).name not in self.bonded_forces.keys():
             self.bonded_forces[type(bonded_force_entity).name] = []
         self.bonded_forces[type(bonded_force_entity).name].append(bonded_force_entity)
+    def deepcopy(self, forcopy):
+        if self.atom1.copied[forcopy] and self.atom2.copied[forcopy]:
+            return ResidueLink(self.atom1.copied[forcopy], self.atom2.copied[forcopy])
         
 class Molecule():
     all = {}
@@ -398,6 +481,18 @@ class Molecule():
     
     def Add_Residue_Link(self, atom1, atom2):
         self.residue_links.append(ResidueLink(atom1, atom2))
+
+    def deepcopy(self):
+        new_molecule = Molecule(self.name)
+        forcopy = hash(str(time.time()))
+        for res in self.residues:
+            new_molecule.Add_Residue(res.deepcopy(forcopy))
+        for link in self.residue_links:
+            new_molecule.residue_links.append(link.deepcopy(forcopy))
+        for res in self.residues:
+            for atom in res.atoms:
+                atom.copied.pop(forcopy)
+        return new_molecule
         
 @Molecule.Set_Save_SPONGE_Input     
 def write_residue(self, prefix, dirname):
@@ -407,6 +502,36 @@ def write_residue(self, prefix, dirname):
     f.write(towrite)
     f.close()
 
+@Molecule.Set_Save_SPONGE_Input     
+def write_coordinate(self, prefix, dirname):
+    towrite = "%d\n"%(len(self.atoms))
+    boxlength = [0, 0, 0, 90, 90, 90]
+    maxi = [-float("inf"), -float("inf"), -float("inf")]
+    mini = [ float("inf"),  float("inf"),  float("inf")]
+    for atom in self.atoms:
+        if atom.x > maxi[0]:
+            maxi[0] = atom.x
+        if atom.y > maxi[1]:
+            maxi[1] = atom.y
+        if atom.z > maxi[2]:
+            maxi[2] = atom.z
+        if atom.x < mini[0]:
+            mini[0] = atom.x
+        if atom.y < mini[1]:
+            mini[1] = atom.y
+        if atom.z < mini[2]:
+            mini[2] = atom.z
+
+    towrite += "\n".join(["%f %f %f"%(atom.x - mini[0] + 3, atom.y - mini[1] + 3, atom.z - mini[2] + 3) for atom in self.atoms])
+    
+    boxlength[0] = maxi[0] - mini[0] + 6
+    boxlength[1] = maxi[1] - mini[1] + 6
+    boxlength[2] = maxi[2] - mini[2] + 6
+    towrite += "\n%f %f %f %f %f %f"%(boxlength[0], boxlength[1], boxlength[2], boxlength[3], boxlength[4], boxlength[5])
+    f = open(os.path.join(dirname, prefix + "_coordinate.txt"),"w")
+    f.write(towrite)
+    f.close()
+
 def Generate_New_Bonded_Force_Type(Type_Name, atoms, properties, Compulsory, Multiple = None):
     
     class BondedForceEntity(Entity):
@@ -496,4 +621,6 @@ def Generate_New_Pairwise_Force_Type(Type_Name, properties):
     
     return PairwiseForceType
 
-from . import LOAD, BUILD
+
+
+from . import LOAD, BUILD, IMPOSE, PROCESS
