@@ -2,6 +2,9 @@ from ... import *
 
 NB14Type = Generate_New_Bonded_Force_Type("nb14", "1-4", {"kLJ":float, "kee": float}, True)
 
+NB14Type.topology_matrix = [[1, -4],
+                                [1, 1]]
+
 @Molecule.Set_Save_SPONGE_Input
 def write_nb14(self, prefix, dirname):
     bonds = []
@@ -12,6 +15,7 @@ def write_nb14(self, prefix, dirname):
                 temp_order = order[::-1]
             else:
                 temp_order = order
+            
             bonds.append("%d %d %f %f"%(self.atom_index[bond.atoms[temp_order[0]]]
             , self.atom_index[bond.atoms[temp_order[1]]], bond.kLJ, bond.kee))
     
